from json import dumps
import click
from rich import print
from rich.prompt import Confirm
from rich_click import RichGroup, RichCommand
import ciscotmg


@click.group(cls=RichGroup)
@click.version_option()
def cli():
    """
    Cisco TMG Matrix CLI\n
    Disclaimer: Cisco makes the data in this tool available for informational purposes.\n
    Cisco does not represent, warrant, or guarantee that it is complete, accurate, or up to date.\n
    This information is subject to change without notice.
    """
    pass


@cli.command(cls=RichCommand)
@click.option("--local", "-l", is_flag=True, help="Local pip upgrade")
@click.option("--force", "-f", is_flag=True, help="Force upgrade, no prompt")
def upgrade(local, force):
    """Upgrade software"""
    svc = ciscotmg.Services()
    online_version = svc.api.check_latest_pypi_version()
    local_version = ciscotmg.Meta.__version__
    if local_version != online_version:
        print(f"New version available: {online_version}")
        if force is False:
            ask = Confirm.ask("Do you wanna upgrade?")
            if ask:
                svc.software.upgrade(online_version, local=local)
    else:
        print(f"Up-to-date version {local_version}")


@cli.command(cls=RichCommand)
@click.argument("pid")
@click.option("--json", "-j", is_flag=True, help="Output in JSON")
@click.option("--output", "-o", type=str, help="Output to file")
def search(json, pid, output):
    """Search PIDs"""
    svc = ciscotmg.Services()
    results = svc.search(pid=pid)
    if json:
        print(dumps(results.json))
    else:
        print(results.cli)
    if output is not None:
        if json:
            ciscotmg.output.write_to_file(dumps(results.json), output)
        else:
            ciscotmg.output.write_to_file(results.cli, output)


@cli.command(cls=RichCommand)
@click.argument("pid")
@click.option("--device", "-n", is_flag=True, help="Network Device")
@click.option("-pf", type=str, help="Product Family (Network Device)")
@click.option("--json", "-j", is_flag=True, help="Output in JSON")
@click.option("--output", "-o", type=str, help="Output to file")
def lookup(device, pf, json, pid, output):
    """Lookup PID"""
    svc = ciscotmg.Services()
    results = svc.lookup(pid=pid, network_device=device, product_family=pf)
    if json:
        print(dumps(results.json))
    else:
        print(results.cli)
    if output is not None:
        if json:
            ciscotmg.output.write_to_file(dumps(results.json), output)
        else:
            ciscotmg.output.write_to_file(results.cli, output)


@cli.command(cls=RichCommand)
def url():
    """Website for Cisco TMG Matrix"""
    tmg_url = "https://tmgmatrix.cisco.com/"
    iop_url = "https://tmgmatrix.cisco.com/iop"
    print(f"Cisco TMG Optics-to-Device Matrix: {tmg_url}")
    print(f"Cisco TMG Optics-to-Optics Matrix: {iop_url}")


if __name__ == "__main__":
    cli()



class Meta:
    """App Information"""

    __title__ = "ciscotmg"
    __description__ = "Private Python SDK for Cisco TMG APIs"
    __url__ = "https://github.com/robertcsapo/ciscotmg"
    __download_url__ = "https://pypi.python.org/pypi/ciscotmg"
    __author__ = "Robert Csapo"
    __email__ = "rcsapo@cisco.com"
    __copyright__ = "Copyright (c) 2022 Cisco Systems, Inc."
    __license__ = "MIT"
    __version__ = "0.2.9"



import requests
from .meta import Meta


class API:
    """HTTP API Request to Cisco TMG API"""

    def __init__(self) -> None:
        """Default settings"""
        self.url = "https://tmgmatrix.cisco.com/public/api"
        self.headers = {
            "accept": "application/json",
            "content-type": "application/json",
            "user-agent": f"{Meta.__title__}-py/v{Meta.__version__}",
        }

    def request(self, path, method, payload=None) -> dict:
        """HTTP requests"""
        if method == "GET":
            response = requests.request(
                method, f"{self.url}{path}", headers=self.headers
            )
        if method == "POST":
            response = requests.request(
                method, f"{self.url}{path}", headers=self.headers, data=payload
            )
        response.raise_for_status()
        try:
            response.json()
        except requests.JSONDecodeError as error_msg:
            raise requests.JSONDecodeError(
                f"Can't decode response from {self.url} - {error_msg}"
            )
        return response.json()

    def check_latest_pypi_version(self) -> str:
        """Check latest version published on pypi"""
        self.url = "https://pypi.python.org/pypi/ciscotmg/json"
        res = self.request(path="", method="GET")
        return res["info"]["version"]



class Sort:
    """Sorting orders for data"""

    @classmethod
    def rows(cls, network_device, pf, data) -> list:
        """Sorts rows based on type"""
        cls.pf = pf
        cls.data = data
        if network_device:
            return cls.network_device()
        return cls.transceiver()

    @classmethod
    def network_device(cls) -> list:
        """Sort Network Device"""
        columns = [
            "PID",
            "Product Family",
            "Data Rate",
            "Form",
            "Max. Reach",
            "Cable Type",
            "Media",
            "Connector Type",
            "Type",
            "Case Temp",
            "DOM HW Capable",
            "OS Minimum",
            "DOM SW",
        ]
        rows = []
        for pids in cls.data["networkDevices"]:
            for p in pids["networkAndTransceiverCompatibility"]:
                for t in p["transceivers"]:
                    t["softReleaseMinVer"] = cls.check_release_notes(
                        t["softReleaseMinVer"]
                    )
                    t["digitalDiagnostic"] = cls.check_dom_support(
                        t["digitalDiagnostic"]
                    )
                    t["temperatureRange"] = cls.add_newline_temperature(
                        t["temperatureRange"]
                    )

                    row = [
                        t["productId"],
                        pids["productFamily"],
                        t["dataRate"],
                        t["formFactor"],
                        t["reach"],
                        t["cableType"],
                        t["media"],
                        t["connectorType"],
                        t["type"],
                        t["temperatureRange"],
                        t["digitalDiagnostic"],
                        t["softReleaseMinVer"],
                        t["softReleaseDOM"],
                    ]

                    if isinstance(cls.pf, list):
                        if pids["productFamily"] in cls.pf:
                            rows.append(row)
                    else:
                        rows.append(row)
        return columns, rows

    @classmethod
    def transceiver(cls) -> list:
        """Sort Transceiver"""
        columns = [
            "PID",
            "Product Family",
            "Data Rate",
            "Form",
            "Max. Reach",
            "Cable Type",
            "Media",
            "Connector Type",
            "Type",
            "Case Temp",
            "DOM HW Capable",
            "OS Minimum",
            "DOM SW",
        ]
        rows = []
        for pids in cls.data["networkDevices"]:
            for p in pids["networkAndTransceiverCompatibility"]:
                for t in p["transceivers"]:
                    t["softReleaseMinVer"] = cls.check_release_notes(
                        t["softReleaseMinVer"]
                    )
                    t["digitalDiagnostic"] = cls.check_dom_support(
                        t["digitalDiagnostic"]
                    )
                    t["temperatureRange"] = cls.add_newline_temperature(
                        t["temperatureRange"]
                    )

                    row = [
                        p["productId"],
                        pids["productFamily"],
                        t["dataRate"],
                        t["formFactor"],
                        t["reach"],
                        t["cableType"],
                        t["media"],
                        t["connectorType"],
                        t["type"],
                        t["temperatureRange"],
                        t["digitalDiagnostic"],
                        t["softReleaseMinVer"],
                        t["softReleaseDOM"],
                    ]

                    if isinstance(cls.pf, list):
                        if pids["productFamily"] in cls.pf:
                            rows.append(row)
                    else:
                        rows.append(row)
        return columns, rows

    @classmethod
    def check_release_notes(cls, data) -> str:
        """Remove long text in table"""
        if "please" in data.lower():
            return "Check RN"
        return data

    @classmethod
    def check_dom_support(cls, data) -> str:
        """Convert text to Yes/No"""
        if data == "Y":
            return "Yes"
        return "No"

    @classmethod
    def add_newline_temperature(cls, data) -> str:
        """Add /n instead of /"""
        if "/" in data:
            res = ""
            data = data.split("/ ")
            for i, d in enumerate(data):
                if i == 0:
                    res = f"{d}"
                else:
                    res = f"{res}\n{d}"
            return res
        return data



from rich.table import Table
from rich.panel import Panel
from rich.console import Console


class Output:
    """CLI Layout"""

    @staticmethod
    def table(title, columns, rows) -> Table:
        """Table Layout"""
        table = Table(title=title)

        if len(rows) == 0:
            """No results"""
            return Panel(title, expand=False)

        for column in columns:
            table.add_column(column, no_wrap=False)
        for row in rows:
            table.add_row(*row)
        return table


def write_to_file(data, filename) -> bool:
    """Write output to file"""
    console = Console(file=filename)
    with console.capture() as capture:
        console.print(data)
    data = capture.get()

    with open(filename, "w", encoding="utf-8") as f:
        f.write(data)
    return True



import unittest
import ciscotmg


class TestLookup(unittest.TestCase):
    """Lookup test"""

    def __init__(self, methodName: str = ...) -> None:
        """Setup instance"""
        super().__init__(methodName)
        self.svc = ciscotmg.Services()

    def test_lookup_success_pid(self):
        """Lookup for working PID"""
        id = 78
        pid = "SFP-10G-LR-S"
        n = "C9300X-12Y"
        pf = "C9300X"
        res = self.svc.lookup(pid=pid, network_device=None, product_family=None)
        t_in_data = [t for t in res.json["filters"][4]["values"] if t["id"] == id]
        t_id = [
            {
                "id": id,
                "name": pid,
                "count": t_in_data[0]["count"],
                "filterChecked": True,
            }
        ]
        n_in_data = [p for p in res.raw["rows"] if p[0] == n]

        self.assertIsInstance(res.json, dict)
        self.assertNotEqual(0, len(res.json))
        self.assertEqual(t_id, t_in_data)
        self.assertEqual(1, len(n_in_data))
        self.assertEqual(pf, n_in_data[0][1])
        self.assertIn("productFamily", res.json["networkDevices"][0])

    def test_lookup_success_pid_with_pf(self):
        """Lookup for working PID"""
        id = 78
        pid = "SFP-10G-LR-S"
        n = "C9300X-12Y"
        pf = "C9300X"
        res = self.svc.lookup(pid=pid, network_device=None, product_family=pf)
        t_in_data = [t for t in res.json["filters"][4]["values"] if t["id"] == id]
        t_id = [
            {
                "id": id,
                "name": pid,
                "count": t_in_data[0]["count"],
                "filterChecked": True,
            }
        ]
        n_in_data = [p for p in res.raw["rows"] if p[0] == n]

        self.assertIsInstance(res.json, dict)
        self.assertNotEqual(0, len(res.json))
        self.assertEqual(t_id, t_in_data)
        self.assertEqual(1, len(n_in_data))
        self.assertEqual(pf, n_in_data[0][1])
        self.assertIn("productFamily", res.json["networkDevices"][0])

    def test_lookup_suggestion_pid(self):
        """Lookup with no exact match and receive suggestion PIDs"""
        pid = "SFP-10G"
        suggestion = {
            "rows": [
                ["SFP-10G-SR", "Yes", "No"],
                ["SFP-10G-SR-S", "Yes", "No"],
                ["SFP-10G-LRM", "Yes", "No"],
            ]
        }
        res = self.svc.lookup(pid=pid, network_device=None, product_family=None)

        self.assertIsInstance(res.json, dict)
        self.assertNotEqual(0, len(res.json))
        self.assertEqual(suggestion, res.raw)

    def test_lookup_failed(self):
        """Lookup failed for pid"""
        pid = "a7c06ea4-5fe1-4bd7-8be6-ciscotmg-py-test"
        res = self.svc.lookup(pid=pid, network_device=None, product_family=None)

        self.assertIsInstance(res.json, dict)
        self.assertEqual(0, len(res.json))
        self.assertEqual(0, len(res.raw))

    def test_lookup_success_pid_network_device(self):
        """Search a working PID"""
        id = 193
        pid = "C9500-32C"
        n_id = [{"id": id, "name": pid, "count": 98, "filterChecked": True}]
        res = self.svc.lookup(pid=pid, network_device=True, product_family=None)
        n_in_data = [n for n in res.json["filters"][7]["values"] if n["id"] == id]

        self.assertIsInstance(res.json, dict)
        self.assertNotEqual(0, len(res.json))
        self.assertEqual(n_id, n_in_data)
        self.assertNotEqual(pid, res.raw["rows"][0][0])
        self.assertEqual(res.json["networkDevices"][0]["productFamily"], "C9500")

    def test_lookup_success_pid_network_device_with_pf(self):
        """Search a working PID with Product Family filter"""
        id = 193
        pid = "C9500-32C"
        pf = "C9500"
        n_id = [{"id": id, "name": pid, "count": 98, "filterChecked": True}]
        res = self.svc.lookup(pid=pid, network_device=True, product_family=pf)
        n_in_data = [n for n in res.json["filters"][7]["values"] if n["id"] == id]

        self.assertIsInstance(res.json, dict)
        self.assertNotEqual(0, len(res.json))
        self.assertEqual(n_id, n_in_data)
        self.assertNotEqual(pid, res.raw["rows"][0][0])
        self.assertEqual(res.json["networkDevices"][0]["productFamily"], pf)

    def test_lookup_failed_transceiver_network_device(self):
        """Search a transceiver PID with Network Device flag"""
        id = 78
        pid = "SFP-10G-LR-S"
        t_id = {"transceiverProductID": [{"id": id, "name": pid}]}
        res = self.svc.lookup(pid=pid, network_device=True, product_family=None)

        self.assertIsInstance(res.json, dict)
        self.assertNotEqual(0, len(res.json))
        self.assertEqual(t_id, res.json)
        self.assertEqual(pid, res.raw["rows"][0][0])
        self.assertEqual("Yes", res.raw["rows"][0][1])
        self.assertEqual("No", res.raw["rows"][0][2])

    def test_lookup_failed_transceiver_with_network_device_and_pf(self):
        """Search a transceiver PID with Network Device flag and Product Family filter"""
        id = 78
        pid = "SFP-10G-LR-S"
        pf = "C9500"
        t_id = {"transceiverProductID": [{"id": id, "name": pid}]}
        res = self.svc.lookup(pid=pid, network_device=True, product_family=pf)

        self.assertIsInstance(res.json, dict)
        self.assertNotEqual(0, len(res.json))
        self.assertEqual(t_id, res.json)
        self.assertEqual(pid, res.raw["rows"][0][0])
        self.assertEqual("Yes", res.raw["rows"][0][1])
        self.assertEqual("No", res.raw["rows"][0][2])

    def test_lookup_failed_pid_missing_network_device(self):
        """Search a Network PID without Network Device flag"""
        id = 193
        pid = "C9500-32C"
        n_id = {"networkDeviceProductID": [{"id": id, "name": pid}]}
        res = self.svc.lookup(pid=pid, network_device=False, product_family=None)
        self.assertNotEqual(0, len(res.json))
        self.assertIsInstance(res.json, dict)
        self.assertEqual(n_id, res.json)
        self.assertEqual(pid, res.raw["rows"][0][0])

    def test_lookup_unique_network_device_with_network_device(self):
        """Search a Network PID that has to be exact with Network Device flag"""
        id = 5182
        pid = "C9300X-48HXN"
        n_id = {"networkDeviceProductID": [{"id": id, "name": pid}]}
        res = self.svc.lookup(pid=pid, network_device=False, product_family=None)
        self.assertNotEqual(0, len(res.json))
        self.assertIsInstance(res.json, dict)
        self.assertEqual(n_id, res.json)
        self.assertEqual(pid, res.raw["rows"][0][0])


if __name__ == "__main__":
    unittest.main(verbosity=3)



import json
from .backend import API
from .output import Output
from .sort import Sort
from .result import Result
from .software import Software
from .meta import Meta

__version__ = Meta.__version__


class Services:
    """Create a instance to call services"""

    def __init__(self) -> None:
        self.res = Result()
        self.api = API()
        self.software = Software()
        self.columns = []
        self.rows = []
        self.res.raw = {}

    def search(self, pid) -> Result:
        """Service for Search"""
        payload = {"suggestionText": pid}
        self.res.json = self.api.request(
            path="/networkdevice/autosuggest",
            method="POST",
            payload=json.dumps(payload),
        )

        self.columns = ["PID", "Transceiver", "Network Device"]
        self.rows = []

        for pid_type, pids in self.res.json.items():
            if "networkDeviceProductID" in pid_type:
                for pid in pids:
                    self.rows.append([pid["name"], "No", "Yes"])
            if "transceiverProductID" in pid_type:
                for pid in pids:
                    self.rows.append([pid["name"], "Yes", "No"])

        self.res.cli = Output.table(
            title="Cisco TMG Search", columns=self.columns, rows=self.rows
        )
        return self.res

    def lookup(self, pid, network_device=False, product_family=None) -> Result:
        """Lookup specific PID"""
        self.pid = pid
        self.title = f"Cisco TMG Lookup\n{self.pid}"
        """ Get PID IDs from Cisco TMG API"""

        if product_family is not None:
            self.product_family = product_family.split(",")
        else:
            self.product_family = None

        pid = self.search(pid=pid)

        if len(pid.json) == 0:
            self.res.cli = Output.table(
                title="Cisco TMG Lookup\nNo unique PID found",
                columns=self.columns,
                rows=self.rows,
            )
            return self.res

        payload = {
            "cableType": [],
            "dataRate": [],
            "formFactor": [],
            "reach": [],
            "searchInput": [""],
            "osType": [],
            "transceiverProductFamily": [],
            "transceiverProductID": [],
            "networkDeviceProductFamily": [],
            "networkDeviceProductID": [],
        }

        if network_device:
            if "networkDeviceProductID" not in pid.json:
                self.res.raw["rows"] = self.rows
                self.res.cli = Output.table(
                    title="Cisco TMG Lookup\nNo network device PID found",
                    columns=self.columns,
                    rows=self.rows,
                )

                return self.res
            """Check if unique Network PID match against lookup PID"""
            pid.json["networkDeviceProductID"] = [
                network_pid
                for network_pid in pid.json["networkDeviceProductID"]
                if network_pid["name"] == self.pid
            ]
            if len(pid.json["networkDeviceProductID"]) != 1:
                self.res.raw["rows"] = self.rows
                self.res.cli = Output.table(
                    title="Cisco TMG Lookup\nNo unique PID found",
                    columns=self.columns,
                    rows=self.rows,
                )
                return self.res
        else:
            """Check if unique SFP PID match against lookup PID"""
            try:
                pid.json["transceiverProductID"] = [
                    transceiver
                    for transceiver in pid.json["transceiverProductID"]
                    if transceiver["name"] == self.pid
                ]
            except KeyError:
                self.res.raw["rows"] = self.rows
                self.res.cli = Output.table(
                    title="Cisco TMG Lookup\nNo Transceiver PID found",
                    columns=self.columns,
                    rows=self.rows,
                )
                return self.res
            if len(pid.json["transceiverProductID"]) != 1:
                self.res.raw["rows"] = self.rows
                self.res.cli = Output.table(
                    title="Cisco TMG Lookup\nNo unique PID found",
                    columns=self.columns,
                    rows=self.rows,
                )

                return self.res

        payload = payload | pid.json

        self.res.json = self.api.request(
            path="/networkdevice/search", method="POST", payload=json.dumps(payload)
        )

        self.columns, self.rows = Sort.rows(
            network_device=network_device, pf=self.product_family, data=self.res.json
        )

        if len(self.rows) == 0:
            self.title = f"{self.title}\n\nNo results"

        self.res.raw["rows"] = self.rows
        self.res.cli = Output.table(
            title=self.title,
            columns=self.columns,
            rows=self.rows,
        )
        return self.res



""" pypi package """
from os import path
from setuptools import setup, find_packages
from ciscotmg.meta import Meta

with open("requirements.txt", "r", encoding="utf-8") as f:
    requirements = f.read().splitlines()

with open(
    path.join(path.abspath(path.dirname(__file__)), "README.md"), encoding="utf-8"
) as f:
    long_description = f.read()

setup(
    name="ciscotmg",
    author=Meta.__author__,
    author_email=Meta.__email__,
    description=Meta.__description__,
    long_description=long_description,
    long_description_content_type="text/markdown",
    url=Meta.__url__,
    version=Meta.__version__,
    packages=find_packages(exclude=["tests", "img"]),
    py_modules=["ciscotmg"],
    install_requires=requirements,
    entry_points="""
        [console_scripts]
        ciscotmg=ciscotmg.cli:cli
    """,
    python_requires=">=3.6",
    license=Meta.__license__ + "; " + Meta.__copyright__,
    classifiers=[
        "Intended Audience :: Developers",
        "Natural Language :: English",
        "Operating System :: OS Independent",
        "Programming Language :: Python",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3.9",
        "Programming Language :: Python :: 3.10",
        "Programming Language :: Python :: Implementation :: CPython",
    ],
)



import unittest
import ciscotmg


class TestSearch(unittest.TestCase):
    """Search test"""

    def __init__(self, methodName: str = ...) -> None:
        """Setup instance"""
        super().__init__(methodName)
        self.svc = ciscotmg.Services()

    def test_search_success_one_pid(self):
        """Search a working PID"""
        id = 78
        pid = "SFP-10G-LR-S"
        res = self.svc.search(pid=pid)

        self.assertIsInstance(res.json, dict)
        self.assertNotEqual(0, len(res.json))
        self.assertIsInstance(res.json["transceiverProductID"], list)
        self.assertIn("id", res.json["transceiverProductID"][0])
        self.assertIn("name", res.json["transceiverProductID"][0])
        self.assertEqual(res.json["transceiverProductID"][0]["id"], id)
        self.assertEqual(res.json["transceiverProductID"][0]["name"], pid)

    def test_search_success_multi_pid(self):
        """Search for multi working PIDs"""
        pid = "SFP-10G"
        res = self.svc.search(pid=pid)

        self.assertIsInstance(res.json, dict)
        self.assertNotEqual(0, len(res.json))
        self.assertIsInstance(res.json["transceiverProductID"], list)
        for p in res.json["transceiverProductID"]:
            self.assertIsInstance(p, dict)
            self.assertIn("id", p)
            self.assertIn("name", p)

    def test_search_failed(self):
        """Search for non-working PID"""
        pid = "a7c06ea4-5fe1-4bd7-8be6-ciscotmg-py-test"
        res = self.svc.search(pid=pid)

        self.assertIsInstance(res.json, dict)
        self.assertEqual(0, len(res.json))


if __name__ == "__main__":
    unittest.main(verbosity=3)



from pip._internal import main as pip
from .meta import Meta


class Software:
    """Manage pypi package"""

    @staticmethod
    def upgrade(version, local=False) -> bool:
        """Upgrade using pip"""
        try:
            if local:
                pip(["install", "-e", "."])
            else:
                pip(["install", f"{Meta.__title__}=={version}"])

        except Exception as error_msg:
            raise Exception(error_msg)

        return True
